﻿using System.Collections;
using System.Collections.Generic;

using DevExpress.XtraEditors;
using DevExpress.XtraEditors.Popup;

namespace TestProject
{
    /// <summary>
    /// 커스텀 토큰 에디터 팝업 폼
    /// </summary>
    public class CustomTokenEditPopupForm : TokenEditPopupForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomTokenEditPopupForm(tokenEdit)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="tokenEdit">토큰 에디터</param>
        public CustomTokenEditPopupForm(TokenEdit tokenEdit) : base(tokenEdit)
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 데이터 소스 구하기 - GetDataSource()

        /// <summary>
        /// 데이터 소스 구하기
        /// </summary>
        /// <returns>데이터 소스</returns>
        protected override IList GetDataSource()
        {
            TokenEditTokenCollection tokenEditTokenCollection = Properties.Tokens;

            TokenEditSelectedItemCollection tokenEditSelectedItemCollection = Properties.SelectedItems;

            List<TokenEditToken> list = new List<TokenEditToken>(Properties.Tokens.Count);

            for(int i = 0; i < tokenEditTokenCollection.Count; i++)
            {
                TokenEditToken tokenEditToken = tokenEditTokenCollection[i];

                if(IsSelected(tokenEditToken))
                {
                    ObjectWrapper objectWrapper = (ObjectWrapper)tokenEditToken.Value;

                    tokenEditToken = new TokenEditToken(tokenEditToken.Description, objectWrapper.Clone());
                }

                list.Add(tokenEditToken);
            }

            return list;
        }

        #endregion
        #region 선택 여부 구하기 - IsSelected(tokenEditToken)

        /// <summary>
        /// 선택 여부 구하기
        /// </summary>
        /// <param name="tokenEditToken">토큰 에디터 토큰</param>
        /// <returns>선택 여부</returns>
        protected bool IsSelected(TokenEditToken tokenEditToken)
        {
            foreach(TokenEditToken item in Properties.SelectedItems)
            {
                if(object.ReferenceEquals(tokenEditToken, item))
                {
                    return true;
                }
            }

            return false;
        }

        #endregion
    }
}