﻿using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 직원
    /// </summary>
    public class Employee : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 속성 변경시 - PropertyChanged

        /// <summary>
        /// 속성 변경시
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 성명
        /// </summary>
        private string name;

        /// <summary>
        /// 토큰 리스트
        /// </summary>
        private IList tokenList;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 성명 - Name

        /// <summary>
        /// 성명
        /// </summary>
        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                if(this.name == value)
                {
                    return;
                }

                this.name = value;

                FirePropertyChangedEvent("Name");
            }
        }

        #endregion
        #region 토큰 리스트 - TokenList

        /// <summary>
        /// 토큰 리스트
        /// </summary>
        [Browsable(false)]
        public IList TokenList
        {
            get
            {
                return this.tokenList;
            }
            set
            {
                this.tokenList = value;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - Employee(name)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="name">성명</param>
        public Employee(string name)
        {
            this.name      = name;
            this.tokenList = new List<string>();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 문자열 구하기 - ToString()

        /// <summary>
        /// 문자열 구하기
        /// </summary>
        /// <returns>문자열</returns>
        public override string ToString()
        {
            return this.name;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChangedEvent(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        protected virtual void FirePropertyChangedEvent(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}