﻿using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;

using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid.Columns;
using DevExpress.XtraGrid.Views.Base;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 업데이트 여부
        /// </summary>
        private bool update = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 그리드 컨트롤을 설정한다.

            this.gridView.OptionsDetail.EnableMasterViewMode = false;
            this.gridView.OptionsSelection.MultiSelect       = true;
            this.gridView.OptionsSelection.MultiSelectMode   = GridMultiSelectMode.CellSelect;
            this.gridView.FocusRectStyle                     = DrawFocusRectStyle.CellFocus;

            this.gridView.OptionsView.ShowGroupPanel = false;

            this.gridView.IndicatorWidth                         = 30;
            this.gridView.OptionsView.ShowIndicator              = true;
            this.gridView.OptionsView.EnableAppearanceEvenRow    = false;
            this.gridView.OptionsView.EnableAppearanceOddRow     = false;
            this.gridView.OptionsBehavior.KeepFocusedRowOnUpdate = false;
            this.gridView.OptionsNavigation.AutoFocusNewRow      = true;

            this.gridView.OptionsBehavior.AutoPopulateColumns      = false;
            this.gridView.OptionsView.ShowColumnHeaders            = true;
            this.gridView.OptionsView.ColumnAutoWidth              = false;
            this.gridView.OptionsMenu.EnableColumnMenu             = false;
            this.gridView.OptionsNavigation.EnterMoveNextColumn    = true;
            this.gridView.OptionsCustomization.AllowColumnMoving   = false;
            this.gridView.OptionsCustomization.AllowColumnResizing = true;
            this.gridView.OptionsCustomization.AllowFilter         = false;
            this.gridView.OptionsCustomization.AllowSort           = false;

            this.gridView.OptionsView.ShowButtonMode                   = ShowButtonModeEnum.ShowForFocusedCell;
            this.gridView.OptionsView.AllowCellMerge                   = false;
            this.gridView.OptionsBehavior.EditorShowMode               = EditorShowMode.Default;
            this.gridView.OptionsBehavior.Editable                     = true;
            this.gridView.OptionsSelection.EnableAppearanceFocusedCell = false;
            this.gridView.OptionsSelection.InvertSelection             = false;
            this.gridView.OptionsHint.ShowCellHints                    = false;
            this.gridView.OptionsClipboard.CopyColumnHeaders           = DefaultBoolean.False;

            #endregion

            #region 성명 컬럼을 설정한다.

            GridColumn nameColumn = new GridColumn();

            nameColumn.Name                    = "nameColumn";
            nameColumn.Caption                 = "Name";
            nameColumn.FieldName               = "Name";
            nameColumn.Width                   = 100;
            nameColumn.OptionsColumn.AllowEdit = false;
            nameColumn.Visible                 = true;
            nameColumn.VisibleIndex            = 0;

            this.gridView.Columns.Add(nameColumn);

            #endregion
            #region 저장소 항목 커스텀 토큰 에디터를 설정한다.

            RepositoryItemCustomTokenEdit repositoryItemCustomTokenEdit = new RepositoryItemCustomTokenEdit();

            repositoryItemCustomTokenEdit.Name              = "repositoryItemMyTokenEdit";
            repositoryItemCustomTokenEdit.AutoHeightMode    = TokenEditAutoHeightMode.RestrictedExpand;
            repositoryItemCustomTokenEdit.EditMode          = TokenEditMode.Manual;
            repositoryItemCustomTokenEdit.EditValueType     = TokenEditValueType.List;
            repositoryItemCustomTokenEdit.MaxExpandLines    = 1;
            repositoryItemCustomTokenEdit.TokenPopulateMode = TokenEditTokenPopupateMode.DisableAutoPopulate;

            repositoryItemCustomTokenEdit.Separators.AddRange(new string[] { "," });

            foreach(KeyValuePair<int, string> keyValuePair in TokenRegistry.GetTokenDictionary())
            {
                repositoryItemCustomTokenEdit.Tokens.AddToken(keyValuePair.Value, new ObjectWrapper(keyValuePair.Key));
            }

            this.gridControl.RepositoryItems.Add(repositoryItemCustomTokenEdit);

            #endregion
            #region 토큰 컬럼을 설정한다.

            GridColumn tokenColumn = new GridColumn();

            tokenColumn.Name         = "tokenColumn";
            tokenColumn.Caption      = "Token";
            tokenColumn.FieldName    = "(none)";
            tokenColumn.Width        = 300;
            tokenColumn.UnboundType  = DevExpress.Data.UnboundColumnType.Object;
            tokenColumn.Visible      = true;
            tokenColumn.VisibleIndex = 1;
            tokenColumn.ColumnEdit   = repositoryItemCustomTokenEdit;

            this.gridView.Columns.Add(tokenColumn);

            #endregion
            #region 이벤트를 설정한다.

            this.gridView.CustomUnboundColumnData              += gridView_CustomUnboundColumnData;
            repositoryItemCustomTokenEdit.ValidateToken        += repositoryItemCustomTokenEdit_ValidateToken;
            repositoryItemCustomTokenEdit.SelectedItemsChanged += repositoryItemCustomTokenEdit_SelectedItemsChanged;

            #endregion

            this.gridControl.DataSource = GetDataSource();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그리드 뷰 커스텀 언바운드 컬럼 데이터 처리하기 - gridView_CustomUnboundColumnData(sender, e)

        /// <summary>
        /// 그리드 뷰 커스텀 언바운드 컬럼 데이터 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_CustomUnboundColumnData(object sender, CustomColumnDataEventArgs e)
        {
            Employee employee = e.Row as Employee;

            if(employee == null)
            {
                return;
            }

            if(e.IsGetData)
            {
                BindingList<ObjectWrapper> bindingList = new BindingList<ObjectWrapper>();

                foreach(string token in employee.TokenList)
                {
                    int id = TokenRegistry.GetID(token);

                    bindingList.Add(new ObjectWrapper(id));
                }

                e.Value = bindingList;
            }
        }

        #endregion
        #region 저장소 항목 커스텀 토큰 에디터 선택 인덱스 변경시 처리하기 - repositoryItemCustomTokenEdit_SelectedItemsChanged(sender, e)

        /// <summary>
        /// 저장소 항목 커스텀 토큰 에디터 선택 인덱스 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void repositoryItemCustomTokenEdit_SelectedItemsChanged(object sender, ListChangedEventArgs e)
        {
            if(this.update)
            {
                return;
            }

            TokenEdit tokenEdit = sender as TokenEdit;

            if(tokenEdit == null)
            {
                return;
            }

            Employee employee = this.gridView.GetFocusedRow() as Employee;

            UpdateToken(tokenEdit, employee);
        }

        #endregion
        #region 저장소 항목 커스텀 토큰 에디터 토큰 검증하기 - repositoryItemCustomTokenEdit_ValidateToken(sender, e)

        /// <summary>
        /// 저장소 항목 커스텀 토큰 에디터 토큰 검증하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void repositoryItemCustomTokenEdit_ValidateToken(object sender, TokenEditValidateTokenEventArgs e)
        {
            int id;

            if(int.TryParse(e.Description, out id))
            {
                string token = TokenRegistry.GetToken(id);

                e.Description = token;
                e.Value       = new ObjectWrapper(id);
                e.IsValid     = true;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 데이터 소스 구하기 - GetDataSource()

        /// <summary>
        /// 데이터 소스 구하기
        /// </summary>
        /// <returns>데이터 소스</returns>
        private BindingList<Employee> GetDataSource()
        {
            BindingList<Employee> list = new BindingList<Employee>();

            list.Add(new Employee("John"));
            list.Add(new Employee("Carl"));
            list.Add(new Employee("Mike"));
            list.Add(new Employee("Kate"));

            return list;
        }

        #endregion
        #region 토큰 업데이트하기 - UpdateToken(tokenEdit, employee)

        /// <summary>
        /// 토큰 업데이트하기
        /// </summary>
        /// <param name="tokenEdit">토큰 에디터</param>
        /// <param name="employee">직원</param>
        private void UpdateToken(TokenEdit tokenEdit, Employee employee)
        {
            if(this.update)
            {
                return;
            }

            this.update = true;

            try
            {
                employee.TokenList.Clear();

                foreach(ObjectWrapper objectWrapper in (IList)tokenEdit.EditValue)
                {
                    string token = TokenRegistry.GetToken(objectWrapper.Source);

                    employee.TokenList.Add(token);
                }

                this.gridView.UpdateCurrentRow();
            }
            finally
            {
                this.update = false;
            }
        }

        #endregion
    }
}